#!/usr/bin/env python

import sys

from federation import topdl
from federation.proof import proof
from federation.remote_service import service_caller
from federation.client_lib import client_opts, exit_with_fault, RPCException,\
	wrangle_standard_options, do_rpc, get_experiment_names, info_format, \
	log_authentication

class operation_opts(client_opts):
    def __init__(self):
        client_opts.__init__(self)
        self.add_option("--experiment_cert", dest="exp_certfile",
		action='callback', callback=self.expand_file, type='str',
                help="experiment certificate file")
        self.add_option("--experiment_name", dest="exp_name",
                type="string", help="human readable experiment name")
	self.add_option('--target', dest='targets', action='append',
		help='targets of the operation')
	self.add_option('--operation', dest='op', help='operation to carry out')
	self.add_option('--param', dest='params', action='append',
		help='parameters to the operation as name=value')

parser = operation_opts()
(opts, args) = parser.parse_args()
try:
    cert, fid, url = wrangle_standard_options(opts)
except RuntimeError, e:
    sys.exit("%s" % e)

if opts.exp_name and opts.exp_certfile:
    sys.exit("Only one of --experiment_cert and --experiment_name permitted")
elif opts.exp_certfile:
    exp_id = { 'fedid': fedid(file=opts.exp_certfile) }
elif opts.exp_name:
    exp_id = { 'localname' : opts.exp_name }
else:
    sys.exit("specify one of --experiment_cert and --experiment_name")

if not opts.op: sys.exit('specifiy an --operation')
if not opts.targets: sys.exit('specify one or more targets')

req = { 'experiment': exp_id, 'operation': opts.op, 'target': opts.targets}
if opts.params:
    param = []
    for p in opts.params:
	try:
	    a, v = p.split('=')
	except ValueError:
	    sys.exit('Parameters must be in name=value: %s' %p)
	param.append({'attribute': a, 'value': v})
    req['parameter'] = param

try:
    resp_dict = do_rpc(req,
	    url, opts.transport, cert, opts.trusted, 
	    serialize_only=opts.serialize_only,
	    tracefile=opts.tracefile,
	    caller=service_caller('Operation'), 
	    responseBody='OperationResponseBody')
except RPCException, e:
    exit_with_fault(e, 'Ftopo', opts)
except RuntimeError, e:
    sys.exit("Error processing RPC: %s" % e)

for s in resp_dict.get('status', []):
    print "Target: %s Message: %s (code: %d)" % \
	    (s.get('target', ''), s.get('description',''), s.get('code', -1))

proof = proof.from_dict(resp_dict.get('proof', {}))
if proof and opts.auth_log:
    log_authentication(opts.auth_log, 'Operation', 'succeeded', proof)
